      program time_testeph
      implicit none
C       this programme is a substantially modified
C       (by Alan W. Irwin, irwin@uvastro.phys.uvic.ca)
C       version of the main programme of the testeph.f JPL fortran
C       programme.  The modifications make this programme work
C       with the specific time ephemeris test file format and time ephemeris
C       interpolation routines of the Irwin & Fukushima time ephemeris
C       export package.

C       time_testeph tests the Irwin & Fukushima time ephemeris reading and
C       Interpolating routine using examples computed from the original
C       time ephemeris.  the reading and interpolating subroutines that are of
C       eventual interest to the user are part of this export package.
C       You must supply "time_test.XXX" to time_testeph, via standard input.
C       "time_test.XXX  is the specially formatted text file that contains
C       the test cases for the ephmeris, teXXX.
C     After the initial identifying text which is concluded by an "EOT" in
C     columns 1-3, the test file contains the following quantities:
C
C         Time Ephemeris Number
C         Julian Year
C         Julian Ephemeris Date
C         icomp (1 or 2 [time dilation integral and derivative) for ikind = 1,
C         1 through 6 (v_E/c components for first 3,
C         Earth acceleration components for last 3]
C         for ikind = 2)
C         ikind (1 for time-dilation integral, 2 for v_E/c components)
C         quantity(icomp, ikind)
C
C     For each test case input, time_testeph
C
C         - computes the corresponding state from data contained 
C           in texxx,
C
C         - compares the two sets,
C
C         - writes an error message if the relative difference between
C           any of the quantities is greater than 10**(-13).
C
C         - writes state and difference information for every 10th
C           test case processed.
C
C
C      This program is written in standard Fortran-77.  
C
C      HOWEVER, there are two parts which are compiler dependent; both have
C      to do with opening and reading a direct-access file.  They are dealt
C      with in the subroutine FSIZERi, i=1,3.  (There are three versions of 
C      this subroutine.
C
C      1) The parameter RECL in the OPEN statement is the number of units per 
C         record.  For some compilers, it is given in bytes; in some, it is given
C         in single precision words.  In the subroutine FSIZER of time_testeph, the
C         parameter NRECL must be set to 4 if RECL is given in bytes; NRECL must 
C         be set to 1 if RECL is given in words.  (If in doubt, use 4 for UNIX;
C         1 for VAX and PC)
C
C      2) Also for the OPEN statement, the program needs to know the exact value
C         of RECL (number of single precision words times NRECL).  Since this 
C         varies from one JPL ephemeris to another, RECL must be determined somehow 
C         and given to the OPEN statement.  There are three methods, depending 
C         upon the compiler.  We (Irwin & Fukushima) only support the
C         linux/unix time ephemeris at the moment so we have only included
C         a time ephemeris modification of the JPL second method, time_fsizer2.
C
C         a)  Use the INQUIRE statement to find the length of the records 
C             automatically before opening the file.  This works for VAX's; 
C             not in UNIX.
C
C         b)  Open the file with an arbitrary value of RECL, read the first record,
C             and use the information on that record to determine the exact value
C             of RECL.  Then, close the file and re-open it with the exact value.
C             This seems to work for UNIX compilers as long as the initial value of
C             RECL is less than the exact value but large enough to get the required 
C             information from the first file.  (For other compilers, this doesn't
C             work since you can open a file only with the exact value of RECL.)
C
C         c)  Hardwire the value of RECL.  This number is NRECL*1652 for DE200, 
C             NRECL*2036 for DE405, and NRECL*1456 for DE406.
C
C             
C
       CHARACTER*6  NAMS(2)
       CHARACTER*3  ALF3

       DOUBLE PRECISION  DEL, tol(6,2)
C        the toleration of errors is 1.d-13 the maximum values
C        which are
C        2 ms ~ 2.d-8 dy for the time ephemeris
C        4.d-10 for its dimensionless derivative (with 4 zeros following for
C        the remainder of 6 components for ikind = 1),
C        1.d-8 + 3.3d-10*sinusoid in v_E^2/c^2 =
C        (1.d-4 + 1.6d-6 *sinusoid)*sinusoid  in first 3 v_E/c components
C        ~ 1.7d-6*sinusoid  dy^{-1} in remaining 3 acceleration components
       data tol/2.d-21, 4.d-23, 4*0.d0, 3*1.d-17, 3*2.d-19/
       DOUBLE PRECISION  ET
       DOUBLE PRECISION  R(6)
       DOUBLE PRECISION  SS(3)
       DOUBLE PRECISION  VALS(2)
       DOUBLE PRECISION  XI
  
       INTEGER  I, icomp, ikind
       INTEGER  LINE
       data line/0/
       INTEGER  NVS
C        print out every test case
       INTEGER  NPT/1/

C      Write a fingerprint to the screen.

       WRITE(*,*) ' Irwin & Fukushima TEST-EPHEMERIS program. ' //
     &   ' Last modified November 1999.'

C      Print the ephemeris constants.

       CALL  time_const (NAMS, VALS, SS, NVS)
       if(nvs.gt.2)
     &   stop 'invalid number of constants in this time ephemeris'

       WRITE (*,'(/3F14.2/200(/2(A8,D24.16)))') SS,
     &   (NAMS(I),VALS(I),I=1,NVS)

C     Skip the file header comments.

   1  READ(*,'(a3)')ALF3
      IF(ALF3 .NE. 'EOT') GO TO 1

      WRITE(*,'(7a)') 
     &  '  line','   jed   ', ' icomp', ' ikind', 
     &  '   Irwin&Fukushima value  ',
     &  '         User value       ',
     &  '         Difference       '
      WRITE(*,*) 

C     Read a value from the test case; skip if not within the time-range
C     of the present version of the ephemeris

   2  READ(*,'(15X,D10.1,2i6, D26.18)',END=9)  
     & ET, icomp, ikind, XI

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

      IF(ET .LT. SS(1)) GO TO 2
      IF(ET .GT. SS(2)) GO TO 2

      CALL  time_pleph ( ET, ikind, R )

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      DEL = ABS(R(icomp)-XI)
      LINE=LINE+1

      if(del.gt.tol(icomp,ikind))
     &  write(*,*) 'Next differences are greater than 1.d-13 '//
     &  ' times the maximum values'
      if(del.gt.tol(icomp,ikind).or.
     &  MOD(LINE,npt) .EQ. 0 )  WRITE(*,200)
     &  LINE,ET,icomp, ikind, XI, r(icomp), del
 200  FORMAT(I6,0pF10.1,2i6, 1p3d26.18)
      GO TO 2
   9  STOP
      END
